<?php
/**
 * Manage Clickio CDN cache
 */

namespace Clickio\CacheControl\Services;

use Clickio as org;
use Clickio\CacheControl as cc;
use Clickio\Utils\SafeAccess;

/**
 * Clickio cdn cache service
 *
 * @package CacheControl\Services
 */
class ClickIoCDN extends cc\ServiceBase
{

    /**
     * Label
     *
     * @var string
     */
    protected $label = "Clickio CDN";

    /**
     * Description
     *
     * @var string
     */
    protected $desc = "Purge Clickio CDN cache";

    /**
     * Cleaner url
     *
     * @var string
     */
    const CLEAN_URL = 'https://all.stage.clickio.com/clickioUtils/clearUrlSyncBatch/';

    /**
     * Key to store transient urls
     *
     * @var string
     */
    const TRANSIENT_KEY = '_clickio_not_purget_urls';

    /**
     * Interface method
     * For more information see method defenition
     *
     * @param array $urllist list of urls
     *
     * @return void
     */
    public function clear(array $urllist)
    {
        if (empty($urllist)) {
            return ;
        }

        $transient_urls = get_transient(static::TRANSIENT_KEY);
        if (!empty($transient_urls)) {
            $urllist = array_merge($urllist, $transient_urls);
            $urllist = array_unique($urllist);
            delete_transient(static::TRANSIENT_KEY);
        }
        set_transient(static::TRANSIENT_KEY, $urllist, 2 * WEEK_IN_SECONDS);

        $params = $this->getRequestParams($urllist);
        $delay = org\Options::get('deffered_purge');
        // we don't need unexpected values like '' or false or NULL
        if (empty($delay)) {
            $delay = 0;
        }
        $req_id = $this->generateRequiestId();
        $url = sprintf("%s?deffered_purge=%s&request_id=%s", static::CLEAN_URL, $delay, $req_id);
        $data = wp_remote_post($url, $params);

        $params['body'] = json_decode($params['body']);
        $this->debug('Purge CDN cache', ["url" => $url, "params" => $params, "response" => $data, "delay" => $delay]);

        if (is_wp_error($data) || $data['response']['code'] >= 400) {
            if (is_wp_error($data)) {
                $err = $data->get_error_message();
            } else {
                $err = sprintf("url: %s\n\nbody: %s\n\nparams: %s", $url, $data['body'], var_export($params, true));
            }
            if (empty($err)) {
                $err = "Undefined error: ".var_export($data, true);
            }
            $urls = "\n\nUrls: ".json_encode($urllist, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
            $this->error($err.$urls);
            return ;
        }

        delete_transient(static::TRANSIENT_KEY);
    }

    /**
     * Build request params
     *
     * @param array $urls list of urls to be purged
     *
     * @return array
     */
    protected function getRequestParams(array $urls): array
    {
        $headers = [
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer '.org\Options::getApplicationKey()
        ];

        $body = $this->buildBody($urls);

        $params = [
            'timeout' => 15,
            'user-agent' => 'WP',
            'followlocation' => true,
            'sslverify' => false,
            'headers' => $headers,
            'body' => wp_json_encode($body)
        ];
        return $params;
    }

    /**
     * Build http body
     *
     * @param array $urls list of urls to be purged
     *
     * @return array
     */
    protected function buildBody(array $urls): array
    {
        $body = [];

        foreach ($urls as $url) {
            if (empty($url)) {
                continue ;
            }
            $params = $this->getUrlRequestParams($url);
            $body = array_merge($body, $params);
        }
        return $body;
    }

    /**
     * Build url struct
     *
     * @param string $url single url
     *
     * @return array
     */
    protected function getUrlRequestParams(string $url): array
    {
        $urls = [];

        $parsed_url = wp_parse_url($url);
        $_server_host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'default');
        $host = SafeAccess::fromArray($parsed_url, 'host', 'string', $_server_host);
        $path = SafeAccess::fromArray($parsed_url, 'path', 'string', '/');
        $query = SafeAccess::fromArray($parsed_url, 'query', 'string', '');
        $parsed_query = [];
        parse_str($query, $parsed_query);

        $purge_all = false;
        if (in_array('purge_all', array_keys($parsed_query))) {
            $purge_all = true;
        }

        $canonical = org\Options::get("purge_canonical");
        if (in_array('purge_canonical', array_keys($parsed_query))) {
            $canonical = 1;
        }

        $mark_deleted = 0;
        if (in_array('deleted', array_keys($parsed_query))) {
            $mark_deleted = 1;
        }

        $urls[] = $this->_getUrlRequestParam($host, $path, $purge_all, $canonical, $mark_deleted);

        if ($purge_all) {
            return $urls;
        }

        if (substr($path, -1)=== '/') {
            $path_without_slash = substr($path, 0, (strlen($path) - 1));
            if (!empty($path_without_slash)) {
                $urls[] = $this->_getUrlRequestParam($host, $path_without_slash, false, $canonical, $mark_deleted);
            }
        }

        return $urls;
    }

    /**
     * Build url struct
     *
     * @param string $host FQDN
     * @param string $path url
     * @param bool $purge_all flag, purge all pages
     * @param int $canonical flag, purge canonical caches
     * @param int $mark_deleted add flag if post was deleted
     *
     * @return array
     */
    private function _getUrlRequestParam(
        string $host,
        string $path,
        bool $purge_all = false,
        int $canonical = 0,
        int $mark_deleted = 0
    ) : array {
        $url_param = [
            'daemon' => org\Options::get("daemon_version", "Master"),
            'domain' =>  $host,
            'url' => $path,
            'desktop' => 1,
            'mobile' => 1
        ];

        if ($purge_all) {
            $url_param['url'] = '/';
            $url_param['canonical'] = $canonical;
            $url_param['all'] = 1;
        }

        if (!empty($mark_deleted)) {
            $url_param['deleted'] = 1;
        }
        return $url_param;
    }

    /**
     * Generate unique sequence
     *
     * @return string
     */
    protected function generateRequiestId()
    {
        $length = random_int(10, 30);
        $bytes = bin2hex(random_bytes($length));
        return md5($bytes);
    }
}