<?php

/**
 * Plugin
 */

namespace Clickio;

use Clickio as org;
use Clickio\Addons\AddonManager;
use Clickio\Authorization\TokenAuthorizationManager;
use Clickio\Cron as cron;
use Clickio\CacheControl as cache;
use Clickio\Db\DatabaseManager;
use Clickio\Integration\IntegrationServiceFactory;
use Clickio\Logger\Interfaces\ILogger;
use Clickio\Logger\Logger;
use Clickio\Prism\Cache\CacheManager;
use Clickio\Prism\Cache\CacheRepo;
use Clickio\RestApi\RestApiFactory;
use Clickio\Utils\CacheUtils;
use Clickio\Utils\FileSystem;
use Clickio\Utils\LocationType;
use Clickio\Utils\SafeAccess;
use Clickio\Utils\Widgets;

/**
 * Plugin entry point
 *
 * @package Clickio
 */
class ClickioPlugin
{
    /**
     * Logger instance
     *
     * @var ILogger
     */
    protected $log = null;

    /**
     * Cache manager instance
     *
     * @var cache\Interfaces\ICacheManager
     */
    protected $cache = null;

    /**
     * Cron manager
     *
     * @var cron\Interfaces\ICronManager
     */
    protected $cron = null;

    /**
     * Cookie name
     *
     * @var string
     */
    protected $preview_key = "cl_debug";

    /**
     * Singletone instance
     *
     * @var ClickioPlugin
     */
    private static $_inst = null;

    /**
     * Post meta key.
     * amphtml error
     *
     * @var string
     */
    const CLICKHOUSE_ERROR_KEY = "clickio_amphtml_error";

    /**
     * Constructor
     *
     * @param ILogger $log Logger instance
     */
    public function __construct(ILogger $log)
    {
        $this->log = $log;
        add_action('activated_plugin', [$this, 'afterActivation']);
        add_action('upgrader_process_complete', [$this, 'afterUpgrade'], 10, 2);
        if (defined('CLICKIO_PLUGIN_NAME')) {
            \register_activation_hook(CLICKIO_PLUGIN_NAME, [$this, 'onActivate']);
            \register_deactivation_hook(CLICKIO_PLUGIN_NAME, [$this, 'onDeactivate']);
        }

        if (!wp_next_scheduled(cron\Events::HOURLY_EVENT)) {
            $this->onActivate();
        }
    }

    /**
     * Activation hook
     * Fires when plugin activated
     *
     * @return void
     */
    public function onActivate()
    {
        $hook = cron\Events::getEvent(cron\Events::HOURLY_EVENT);
        if (!empty($hook)) {
            wp_clear_scheduled_hook($hook['name']);
            wp_schedule_event(time(), $hook['interval'], $hook['name']);
        }
        org\Options::loadOptions();
        org\Options::loadRemoteOptions();

        $auth = TokenAuthorizationManager::create();
        $app_key = $auth->generateAplicationKey();
        org\Options::setApplicationKey($app_key);
        org\Options::save();

        DatabaseManager::updateIfRequired();

        // $this->setupAccelerator();
    }

    /**
     * Hook handler.
     * Fire after plugin updated
     *
     * @param mixed $cls upgrader instance
     * @param array $options upgrade options
     *
     * @return void
     */
    public function afterUpgrade($cls, $options)
    {
        $this->log->debug("Upgrade process complete", $options);
        if ($options['action'] === 'update' && $options['type'] === 'plugin' && isset( $options['plugins'])) {
            foreach ($options['plugins'] as $plugin) {

                if ($plugin == CLICKIO_PLUGIN_NAME) {
                    $this->log->info("Plugin updated");
                    $this->onActivate();
                    break;
                }
            }
        }
    }

    /**
     * Plugin activation hook
     * Fire after plugin been activated
     *
     * @param string $plugin plugin name
     *
     * @return void
     */
    public function afterActivation($plugin)
    {
        if ($plugin == CLICKIO_PLUGIN_NAME) {
            $auth = TokenAuthorizationManager::create();
            $app_key = $auth->generateAplicationKey();
            org\Options::setApplicationKey($app_key);
        }
    }

    /**
     * Deactivation hook
     * Fires when plugin deactivated
     *
     * @return void
     */
    public function onDeactivate()
    {
        wp_clear_scheduled_hook(cron\Events::HOURLY_EVENT);
        $symlink = sprintf("%s/accelerator.php", WPMU_PLUGIN_DIR);
        if (is_link($symlink)) {
            @unlink($symlink);
        }
    }

    /**
     * Start listning events
     *
     * @return void
     */
    public function setupEventListners()
    {
        add_action('init', [$this, 'wpInit'], 25);
        add_action('admin_init', [$this, 'initAdmin']);
        add_action('admin_menu', [$this, 'initAdminMenu']);
        add_action('admin_enqueue_scripts', [$this, 'staticFiles']);
        add_action('wp_enqueue_scripts', [$this, 'staticFiles']);
        add_action('admin_enqueue_scripts', [$this, 'adminStaticFiles']);
        add_action("shutdown", [CacheManager::class, 'saveBuffer']);
        add_action('wp', [$this, 'onWpReady']);

        $rest = RestApiFactory::createManager('default');
        add_action('rest_api_init', [$rest, "registerRestRoutes"]);

        add_filter("plugin_action_links_".CLICKIO_PLUGIN_NAME, [$this, 'addPluginLinks']);

        if (!LocationType::isGetId() && Options::get('log_widgets') == 1) {
            add_filter("dynamic_sidebar", [Widgets::class, "lookupPageWidgets"]);
        }

        add_filter("get_pagenum_link", [$this, "omitLinksJunk"]);

        $ampforwp = IntegrationServiceFactory::getService("ampforwp");
        $ampforwp::removeGetIdFromLinks();

        $qadsens = IntegrationServiceFactory::getService("qadsens");
        $qadsens::registerSaveOptionsHandler();

        $discuz = IntegrationServiceFactory::getService('wpdiscuz');
        $discuz::setupCacheCleaners();
    }

    /**
     * Fires when global WP object is ready
     *
     * @return void
     */
    public function onWpReady()
    {
        $addons_manager = AddonManager::create();
        $addons_manager->loadAddons();
    }

    /**
     * Hook 'init'
     *
     * @return void
     */
    public function wpInit()
    {
        $url = SafeAccess::fromArray($_SERVER, 'REQUEST_URI', 'string', '');
        $status = CacheUtils::getCacheStatus();
        if ($status) {
            $repo = CacheRepo::getInstance();
            $cache = $repo->get($url);
            if (!empty($cache)) {
                CacheUtils::setCacheStatusHeader(true);
                $this->log->debug("Page from cache", ["url" => $url]);
                echo $cache;
                exit(0);
            } else {
                $mngr = CacheManager::make();
                $mngr->startCache();
            }
        }

        $integration = Options::get('integration_scheme');

        if ($integration == 'cms') {
            $debug = CacheUtils::getCacheStatusArray();
            $this->log->debug("Cache permissions", ["url" => $url, "permissions" => $debug]);
        }
    }

    /**
     * Add custom buttons to plugins page
     *
     * @param array $links links to add to plugin row
     *
     * @return array
     */
    public function addPluginLinks(array $links)
    {
        $url = esc_url(
            add_query_arg(
                'page',
                org\Options::OPT_KEY,
                get_admin_url() . 'admin.php'
            )
        );
        $link = sprintf('<a href="%s">%s</a>', $url, "Settings");
        $links['settings'] = $link;
        return $links;
    }

    /**
     * Init plugin
     *
     * @return void
     */
    // public function initPlugin()
    // {
    //     global $wp_post_types;
    //     $customtypes = org\Options::get('customtypes', []);
    //     $post_type_names = explode(',', $customtypes);
    //     foreach ($post_type_names as $post_type_name) {
    //         if (isset($wp_post_types[$post_type_name])) {
    //             $wp_post_types[$post_type_name]->show_in_rest = true;
    //             // Optionally customize the rest_base or controller class
    //             $wp_post_types[$post_type_name]->rest_base = $post_type_name;
    //             $wp_post_types[$post_type_name]->rest_controller_class = 'WP_REST_Posts_Controller';
    //         }
    //     }
    // }

    /**
     * Fires as an admin screen or script is being initialized.
     * Deactivate self and then print admin notice
     *
     * @return void
     */
    public function initAdmin()
    {
        if (version_compare(get_bloginfo('version'), '5.0', '<')) {
            deactivate_plugins(CLICKIO_PLUGIN_NAME);
            add_action('admin_notices', [$this, 'requireWpVersionNotice']);
            return;
        }

        register_setting(
            org\Options::OPT_KEY,
            org\Options::OPT_KEY,
            [
                "sanitize_callback" => [org\Options::class, 'validateOptions']
            ]
        );
    }

    /**
     * Fires before the administration menu loads in the admin.
     *
     * @return void
     */
    public function initAdminMenu()
    {
        add_options_page(
            'Clickio',
            'Clickio',
            'manage_options',
            org\Options::OPT_KEY,
            [$this, 'displaySettingsPage']
        );
    }

    /**
     * Admin notice
     * Prints incompatible WP version
     *
     * @return void
     */
    public function requireWpVersionNotice()
    {
        echo "<div id='message' class='notice notice-error is-dismissible'>
            <p>".
                sprintf(
                    'Clickio Plugin requires WordPress 5.0 or greater. Please <a href="%1$s">upgrade WordPress</a>.',
                    esc_url(admin_url('update-core.php'))
                ).
            "</p>
        </div>";
    }

    /**
     * Singletone
     *
     * @param ?\ILogger $log Logger instance
     * @param ?\ICacheManager $cache_manager Cache manager instance
     * @param ?\ICronManager $cron Cron tasks manager
     *
     * @return self
     */
    public static function getInstance($log = null, $cache_manager = null, $cron = null)
    {
        if (empty(static::$_inst)) {

            if ($log) {
                $logger = $log;
            } else {
                $log_name = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
                $logger = Logger::getLogger($log_name);
            }

            if ($cache_manager) {
                $cache = $cache_manager;
            } else {
                $cache = new cache\CacheManager($logger);
            }

            if ($cron) {
                $cron_manager = $cron;
            } else {
                $cron_manager = new cron\CronTaskManager($logger);
            }

            static::$_inst = new static($logger);
            static::$_inst->setCacheControll($cache);
            static::$_inst->setCronTaskManager($cron_manager);
        }

        return static::$_inst;
    }

    /**
     * Add custom static to all pages
     *
     * @return array
     */
    public function staticFiles()
    {
        if (is_user_logged_in() && is_admin_bar_showing()) {
            $url_css = plugins_url("style.css", CLICKIO_PLUGIN_DIR.'/src/static/style.css'); //dirty hack
            wp_register_style('clickio_cdn', $url_css, false, filemtime(CLICKIO_PLUGIN_DIR.'/src/static/style.css'));
            wp_enqueue_style('clickio_cdn');
        }
    }

    /**
     * Add custom statics to admin pages
     *
     * @param string $prefix admin page identifier
     *
     * @return void
     */
    public function adminStaticFiles($prefix)
    {
        // load options js only on own settings page

        if ($prefix == 'settings_page_clickio_opt') {
            $url_js = plugins_url("options.js", CLICKIO_PLUGIN_DIR.'/src/static/options.js'); //dirty hack

            wp_register_script('clickio_cdn', $url_js, false, CLICKIO_PRISM_VERSION, true);
            wp_enqueue_script('clickio_cdn');
        }
    }

    /**
     * Toggle preview mode
     *
     * @return int
     */
    public function getPreviewMode(): int
    {
        if (!empty($_COOKIE[$this->preview_key])) {
            return (int) $_COOKIE[$this->preview_key];
        } else if (!empty($_REQUEST[$this->preview_key])) {
            return (int) $_REQUEST[$this->preview_key];
        } else {
            return 0;
        }
    }

    /**
     * Getter.
     * Get logger
     *
     * @return string
     */
    public function getLogger(): ILogger
    {
        return $this->log;
    }

    /**
     * Getter.
     * Get cache manager
     *
     * @return array
     */
    public function getCache(): cache\Interfaces\ICacheManager
    {
        return $this->cache;
    }

    /**
     * Getter.
     * Get cron manager
     *
     * @return array
     */
    public function getCron(): cron\Interfaces\ICronManager
    {
        return $this->cron;
    }

    /**
     * Setter.
     * Set cache manager
     *
     * @param cache\Interfaces\ICacheManager $cache some cache manager
     *
     * @return void
     */
    public function setCacheControll(cache\Interfaces\ICacheManager $cache)
    {
        $this->cache = $cache;
    }

    /**
     * Setter.
     * Set cron manager
     *
     * @param \ICronManager $cron some cron manager
     *
     * @return void
     */
    public function setCronTaskManager(cron\Interfaces\ICronManager $cron)
    {
        $this->cron = $cron;
    }

    /**
     * Display settings page
     *
     * @return void
     */
    public function displaySettingsPage()
    {
        include CLICKIO_PLUGIN_DIR."/clickio_settings.php";
    }

    /**
     * When wp cron is disabled, check and run only own tasks
     *
     * @return void
     */
    // protected function checkAndRunEvents()
    // {
    //     $now = time();

    //     $events = cron\Events::getEvents();

    //     $opts = [
    //         'http'=>[
    //             'method'=>"GET",
    //             'header'=> sprintf("Host: %s\r\n", $_SERVER['SERVER_NAME']),
    //             'timeout' => 1
    //         ]
    //     ];

    //     foreach ($events as $event) {
    //         $dt = wp_next_scheduled($event['name']);
    //         if (!empty($dt) && $dt <= $now) {
    //             $is_locked = cron\CronTaskManager::lockTask($event['name']);
    //             if ($is_locked) {
    //                 if (wp_unschedule_event($dt, $event['name'])) {
    //                     wp_reschedule_event($dt, $event['interval'], $event['name']);

    //                     $query = ["task" => $event['name']];
    //                     $context = stream_context_create($opts);
    //                     $port = $_SERVER['SERVER_PORT'];
    //                     $proto = $port == 80? 'http' : 'https';
    //                     $link = sprintf("%s://127.0.0.1/wp-json/clickio/run_deffered?%s", $proto, http_build_query($query));
    //                     try {
    //                         $resp = file_get_contents($link, false, $context);
    //                     } catch (\Exception $err) {
    //                         $this->log->debug($err->getMessage(), $err->getTrace());
    //                         $resp = null;
    //                     }
    //                     if (empty($resp)) {
    //                         cron\CronTaskManager::releaseTask($event['name']);
    //                     }
    //                 }
    //             }
    //         }
    //     }
    // }

    /**
     * Authentication
     *
     * @param string $key appkey
     *
     * @return bool
     */
    public function authenticate(string $key): bool
    {
        $stored_key = Options::getApplicationKey();
        return $stored_key == $key;
    }

    /**
     * Get url to wp rest api
     *
     * @return string
     */
    public static function getRestUrl(): string
    {
        $domain = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        return sprintf('https://%s/wp-json/wp/v2', $domain);
    }

    /**
     * Disable pagination
     *
     * @param mixed $pages list of pages
     *
     * @return array
     */
    public function disablePagination($pages)
    {
        $disable = Options::get('disable_pages', 0);
        if (!empty($disable)) {
            $full_page = implode('', $pages);
            $pages = [$full_page];
        }
        return $pages;
    }

    /**
     * Remove unnecessary query parameter from links
     *
     * @param string $link target link
     *
     * @return string
     */
    public function omitLinksJunk($link)
    {
        $url = parse_url($link);
        $query = SafeAccess::fromArray($url, 'query', 'string', '');
        if (array_key_exists('query', $url)) {
            $query = explode('&', $url['query']);
            $query = array_filter(
                $query,
                function($val) {
                    if (strpos($val, 'get_id') === false && strpos($val, 'anticache') === false) {
                        return $val;
                    }
                }
            );
            $query = implode('&', $query);
        }
        if (!empty($query)) {
            $query = "?".$query;
        }
        return sprintf("%s://%s%s%s", $url['scheme'], $url['host'], $url['path'], $query);
    }

    /**
     * Setup accelerator
     *
     * @return bool
     */
    public function setupAccelerator()
    {
        $this->log->debug("Setup accelerator", []);
        $accelerator_file = sprintf('%s/src/Accelerator/accelerator.php', CLICKIO_PLUGIN_DIR);
        $symlink = sprintf("%s/accelerator.php", WPMU_PLUGIN_DIR);

        $success = FileSystem::createSymlink($accelerator_file, $symlink);
        if (!$success) {
            $this->log->error("Unable to setup acclerator");
        }
        return $success;
    }

    /**
     * Uninstall Clickio Accelerator
     *
     * @return void
     */
    public function uninstallAccelerator()
    {
        $target = sprintf("%s/accelerator.php", WPMU_PLUGIN_DIR);
        $this->log->debug('Uninstall Accelerator', ["target" => $target]);

        FileSystem::removeFile($target);
    }

    /**
     * Redirect to canonical url to avoid blank pages in Prism
     *
     * @param bool $do_redirect when true it will break plugin execution
     *                          and returns 301 response code
     *
     * @return void|string
     */
    public function canonicalRedirect(bool $do_redirect = true)
    {
        if (LocationType::is404()) {
            return ;
        }

        $https = SafeAccess::fromArray($_SERVER, 'HTTPS', 'mixed', 'on');
        $server_port = SafeAccess::fromArray($_SERVER, 'SERVER_PORT', 'mixed', 443);
        $domainName = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        $request_uri = SafeAccess::fromArray($_SERVER, 'REQUEST_URI', 'string', '/');
        $protocol = (!empty($https) && $https !== 'off' || $server_port == 443) ? "https://" : "http://";

        $requested_url = $protocol.$domainName.$request_uri;
        $parsed_url = parse_url($requested_url);
        $query = array_key_exists('query', $parsed_url)? '?'.$parsed_url['query'] : '';
        $path = '';

        if (substr($parsed_url['path'], -1) == '/') {
            // without trailing slash
            $path = substr($parsed_url['path'], 0, (strlen($parsed_url['path']) - 1)).$query;
        } else {
            $path = $parsed_url['path'].'/'.$query;
        }
        $alt_url = sprintf("%s://%s%s", $parsed_url['scheme'], $parsed_url['host'], $path);

        if (function_exists('redirect_canonical')) {
            $redirect_to = redirect_canonical($requested_url, false);
            if (!empty($redirect_to) && !in_array($redirect_to, [$requested_url, $alt_url])) {
                $this->log->debug("Canonical redirect", ["request_url" => $requested_url, "redirect_url" => $redirect_to]);
                return redirect_canonical($requested_url, $do_redirect);
            }
        }
    }

    /**
     * Plugin User-Agent
     *
     * @return string
     */
    public static function getPluginUA(): string
    {
        global $wp_version;
        $wp = $wp_version;
        $host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        $plugin = CLICKIO_PRISM_VERSION;

        $ua_pattern = "Clickio-Prism/%s (WordPress/%s; %s)";
        return sprintf($ua_pattern, $plugin, $wp, $host);
    }
}
