<?php

/**
 * Prism manager
 */

namespace Clickio\Prism;

use Clickio\CacheControl\CacheServiceFactory;
use Clickio\ClickioPlugin;
use Clickio\Logger\Interfaces\ILogger;
use Clickio\Logger\Logger;
use Clickio\Options;
use Clickio\Prism\Cache\CacheFactory;
use Clickio\Prism\Cache\CacheRepo;
use Clickio\Prism\Cache\Engine\Files;
use Clickio\Prism\Cache\Interfaces\ICacheRepo;
use Clickio\Prism\Cache\Interfaces\ICacheRepository;
use Clickio\Request\Request;
use Clickio\Utils\LocationType;
use Clickio\Utils\SafeAccess;
use Exception;

/**
 * Clickio prism manager
 *
 * @package Prism
 */
class PrismManager
{
    /**
     * Singletone container
     *
     * @var self
     */
    protected static $inst = null;

    /**
     * Logger instance
     *
     * @var ILogger
     */
    protected $log = null;

    /**
     * Cache repository instance
     *
     * @var ICacheRepo
     */
    protected $cache = null;

    /**
     * Constructor
     *
     * @param ILogger $logger logger instance
     * @param ICacheRepo $cache cache repo
     */
    public function __construct(ILogger $logger, ICacheRepo $cache)
    {
        $this->log = $logger;
        $this->cache = $cache;
    }

    /**
     * Factory method
     *
     * @param ?ICacheRepo $repo cache repository
     *
     * @return self
     */
    public static function getInstance($repo = null): self
    {
        if (empty(static::$inst)) {
            $host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
            $logger = Logger::getLogger($host);

            if (empty($repo)) {
                $repo = CacheRepo::getInstance();
            }
            static::$inst = new static($logger, $repo);
        }

        return static::$inst;
    }

    /**
     * Static alias for getPrism method
     *
     * @return string
     */
    public static function getPrismPage(): string
    {
        $obj = static::getInstance();
        return $obj->getPrism();
    }

    /**
     * Get prism daemon host
     *
     * @return string
     */
    private function _getDaemonHost(): string
    {
        $daemon_cookie = SafeAccess::fromArray($_COOKIE, 'cldaemon', 'string', '');
        $daemon_cookie = ucfirst($daemon_cookie);
        $domain = Options::getPwaHost($daemon_cookie);
        return sprintf("https://%s", $domain);
    }

    /**
     * Get prism daemon endpoint
     *
     * @return string
     */
    protected function getDaemonUrl(): string
    {
        $is_swipe = SafeAccess::fromArray($_SERVER, 'HTTP_SWIPE', 'string', 0);
        $host = $this->_getDaemonHost();
        if ($is_swipe) {
            $url = sprintf("%s/cl-shadow", $host);
        } else {
            $url = sprintf("%s/a", $host);
        }
        $raw_url = SafeAccess::fromArray($_SERVER, "REQUEST_URI", 'string', '');
        $parsed_url = wp_parse_url($raw_url);
        $path = SafeAccess::fromArray($parsed_url, 'path', 'string', '/');
        return sprintf("%s%s", $url, $path);
    }

    /**
     * Get request parameters for the Prism daemon
     *
     * @return array
     */
    protected function getDaemonParams(): array
    {
        $params = [
            "lx_wp" => 1,
            "domain" => SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost'),
        ];

        if (LocationType::isPage()) {
            $params["lx_wp_type"] = 'pages';
            $params["lx_wp_id"] = get_the_ID();
        }

        if (LocationType::isArchive()) {
            $params["wp_is_category"] = 1;
        }

        $is_https = SafeAccess::fromArray($_SERVER, 'HTTPS', 'string', 'off');
        if ($is_https == 'on' ) {
            $params['original_scheme'] = 'https';
        }

        $raw_url = SafeAccess::fromArray($_SERVER, "REQUEST_URI", 'string', '');
        $parsed_url = wp_parse_url($raw_url);
        $raw_query = SafeAccess::fromArray($parsed_url, 'query', 'string', '');
        foreach (explode("&", $raw_query) as $query_items_list) {
            $exploded = explode("=", $query_items_list);
            $key = SafeAccess::fromArray($exploded, 0, 'string', '');
            $value = SafeAccess::fromArray($exploded, 1, 'string', '');
            if (!empty($key) && !empty($value)) {
                $params[$key] = $value;
            }
        }
        return $params;
    }

    /**
     * Make a request to the daemon
     *
     * @return string
     */
    private function _getPrism(): string
    {
        $pwaHost = $this->getDaemonUrl();
        $params = $this->getDaemonParams();
        $req = Request::create(["signed" => false]);
        $resp = $req->get($pwaHost, $params);
        $cl_debug = SafeAccess::fromArray($_REQUEST, 'cl_debug', 'string', '0');
        if (!empty($cl_debug) && $cl_debug > 2) {
            $this->_outputDebug(
                [
                "url" => $pwaHost."?".http_build_query($params),
                "request" => $req,
                "response" => $resp
                ]
            );
        }
        if ($resp->response >= 400) {
            $headers = $resp->headers;
            throw new \Exception($headers['x-error']);
        }
        return $resp->body;
    }

    /**
     * Get prism page
     *
     * @return string
     */
    public function getPrism(): string
    {

        try {
            $prism = $this->_getPrism();
            return $prism;
        } catch (Exception $err) {
            $this->log->error($err->getMessage());
            header('x-clickio-prism-error: '.$err->getMessage());
            return '';
        }
    }

    /**
     * Format debug output
     *
     * @param array array $data debug info
     *
     * @return void
     */
    private function _outputDebug(array $data)
    {
        echo "<div><h3>Debug:</h3></div>";
        echo "<pre>";
        echo htmlentities(var_export($data, true));
        echo "</pre>";
        die();
    }
}