<?php
/**
 * Abstract rest api action
 */

namespace Clickio\RestApi;

use Clickio\Authorization\TokenAuthorizationManager;
use Clickio\Logger as log;
use Clickio\Utils\SafeAccess;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Base class
 *
 * @package RestApi
 */
abstract class BaseRestAction
{
    /**
     * Logger instance
     *
     * @var log\Interfaces\ILogger
     */
    protected $log = null;

    /**
     * Http request
     *
     * @var WP_REST_Request
     */
    protected $request = null;

    /**
     * Constructor
     *
     * @param log\Interfaces\ILogger $logger Logger instance
     * @param WP_REST_Request $request WP http request
     */
    public function __construct(log\Interfaces\ILogger $logger, WP_REST_Request $request)
    {
        $this->log = $logger;
        $this->request = $request;
    }

    /**
     * Write info message
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function notice(string $msg)
    {
        $msg = sprintf("Rest action: %s: %s\n", static::class, $msg);
        $this->log->notice($msg);
    }

    /**
     * Write debug message
     *
     * @param string $msg log message
     * @param array $debug debug info
     *
     * @return void
     */
    protected function debug(string $msg, array $debug)
    {
        $msg = sprintf("Rest action: %s: %s\n", static::class, $msg);
        $this->log->debug($msg, $debug);
    }

    /**
     * Extract payload for protected endpoints
     *
     * @return array
     */
    protected function getPayload(): array
    {
        $body = $this->request->get_body();
        $decoded = json_decode($body, true);
        $payload = SafeAccess::fromArray($decoded, "payload", "string", "{}");
        return json_decode($payload, true);
    }

    /**
     * Dispatcher
     * Call rest action method by http request method
     *
     * @param WP_REST_Request $request http request
     *
     * @return mixed
     */
    public static function dispatch(WP_REST_Request $request)
    {
        $host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        $logger = log\Logger::getLogger($host);
        $obj = new static($logger, $request);
        $method = strtolower($request->get_method());
        if (method_exists($obj, $method)) {
            return call_user_func([$obj, $method]);
        }
        return new WP_REST_Response(null, 501);
    }

    /**
     * Validate message to protected namespace
     *
     * @param WP_REST_Request $request http request
     *
     * @return bool
     */
    public static function hasPermissions(WP_REST_Request $request): bool
    {
        $body = $request->get_body();
        $decoded = json_decode($body, true);
        $payload = SafeAccess::fromArray($decoded, "payload", "string", "{}");
        $token = SafeAccess::fromArray($decoded, "token", "string", "");
        $auth_serv = TokenAuthorizationManager::create();
        return $auth_serv->validate($payload, $token);
    }
}