<?php

/**
 * File system utils
 */

namespace Clickio\Utils;

use Clickio\Logger\Logger;
use Clickio\Options;

/**
 * File system utils
 *
 * @package Utils
 */
class FileSystem
{
    /**
     * Recursively create the full path
     *
     * @param string $dir path to directory
     *
     * @return bool
     */
    public static function makeDir(string $dir): bool
    {
        if (is_dir($dir)) {
            return true;
        }

        mkdir($dir, 0777, true);
        return is_dir($dir);
    }

    /**
     * Open file write mode
     * Recursively create the full path if it doesn't exist
     *
     * @param string $path full file path
     * @param string $mode write mode
     *
     * @return mixed
     */
    public static function openFileOnWrite(string $path, string $mode)
    {
        $path = sprintf("%s/%s", ABSPATH, $path);

        $dir = dirname($path);

        if (static::makeDir($dir)) {
            $fp = fopen($path, $mode);
            return $fp;
        }

        return null;
    }

    /**
     * Recursive remove dir
     *
     * @param string $src path to be removed
     *
     * @return bool
     */
    public static function rrmdir($src): bool
    {
        $dir = opendir($src);
        if (!$dir) {
            return false;
        }

        while (false !== ($file = readdir($dir))) {
            if (in_array($file, ['..', '.'])) {
                continue;
            }

            $full = implode(DIRECTORY_SEPARATOR, [$src, $file]);
            if (is_dir($full)) {
                static::rrmdir($full);
            } else {
                unlink($full);
            }
        }
        closedir($dir);
        rmdir($src);
        return true;
    }

    /**
     * Create symlink
     * Works like "/bin/bash ln -s /path/to/source /path/to/dest"
     *
     * @param string $target symlink points to
     * @param string $name symlink location
     *
     * @return bool
     */
    public static function createSymlink(string $target, string $name): bool
    {
        if (!is_readable($target)) {
            return false;
        }

        if (@readlink($name)) {
            return true;
        }

        if (static::_functionDisabled("symlink")) {
            $host = SafeAccess::fromArray($_SERVER, "HTTP_HOST", "string", "localhost");
            $logger = Logger::getLogger($host);
            $logger->warning("The function \"symlink\" is disabled by php configuration.");
            return false;
        }
        $symlink = @symlink($target, $name);
        if (empty($symlink)) {
            $symlink = false;
        }
        return $symlink;
    }

    /**
     * Check if the function is disabled
     *
     * @param string $func_name function name
     *
     * @return bool
     */
    private static function _functionDisabled(string $func_name): bool
    {
        $disabled_functions = ini_get('disable_functions');
        $disabled_list = preg_split('/,\s*/', $disabled_functions);
        return in_array($func_name, $disabled_list);
    }

    /**
     * Remove file
     *
     * @param string $path file path
     *
     * @return bool
     */
    public static function removeFile(string $path): bool
    {
        if (!@is_readable($path)) {
            return false;
        }

        if (!@is_file($path)) {
            return false;
        }

        return @unlink($path);
    }
}